﻿#Region " Imports "
Imports System.Runtime.Serialization
Imports System.Security.Permissions
Imports System.Data.Odbc
Imports System.IO
Imports System.ComponentModel
Imports System.Web.UI.WebControls
Imports System.Web

Imports Infragistics.Web.UI
Imports Infragistics.Web.UI.Framework
Imports Infragistics.Web.UI.GridControls

Imports Data.Tsql
Imports Data

#End Region
<Serializable()>
Public Class Settings
    Inherits Global.Grid.BaseObject

#Region " Constants "
    Private Const HIDDEN_FIELDNAME_PREFIX As String = "Hidden "
    Private Const IMAGES_WEBTAB_PATH As String = "./images/webtab/"
    Private Const CHECKBOX_NOT_COMPLETED As String = "Not completed."
    Private Const CHECKBOX_COMPLETED As String = "Completed."
    Private Const ALERT_OVERDUE As String = "Over Due"
    Private Const ALERT_DUE As String = "Due"

    ' DEFAULTS
    Private Const DEF_ID As String = ""
    Private Const DEF_TYPE As GridType = GridType.NotSet
    Private Const DEF_RECORD_TYPE As Integer = 0
    Private Const DEF_PAGE_INDEX As Integer = 1
    Private Const DEF_PAGE_SIZE As Integer = 10
    Private Const DEF_COLUMN_WIDTH As Integer = 150
    Private Const DEF_IMG_COLUMN_WIDTH As Integer = 20

    'PROPERTY NAMES
    Private Const PROP_GRIDID_STRING As String = "GridId"
    Private Const PROP_GRIDTYPE_STRING As String = "GridType"
    Private Const PROP_RECORDTYPE_STRING As String = "RecordType"
    Private Const PROP_SELECT_STRING As String = "SelectString"
    Private Const PROP_TABLE_NAME As String = "TableName"
    Private Const PROP_WHERE_STRING As String = "WhereString"
    Private Const PROP_ORDERBY_STRING As String = "OrderByString"
    Private Const PROP_GROUPBY_STRING As String = "GroupByString"
    Private Const PROP_HIDDENFIELDS_STRING As String = "HiddenFields"
    Private Const PROP_GROUPBY_SETTINGS_STRING As String = "GroupBySettings"
    Private Const PROP_SELECTEDVIEWINDEX_STRING As String = "SelectedViewIndex"
    Private Const PROP_SELECTEDVIEWNAME_STRING As String = "SelectedViewName"
    Private Const PROP_PAGEINDEX_STRING As String = "PageIndex"
    Private Const PROP_PAGESIZE_STRING As String = "PageSize"
    Private Const PROP_RECORDCOUNT_STRING As String = "RecordCount"
#End Region

#Region " Declarations "
    Private _id As String
    Private _type As GridType
    Private _recordType As Integer
    Private _selectString As String
    Private _tableName As String
    Private _whereString As String
    Private _orderbyString As String
    Private _groupbyString As String
    Private _hiddenfieldsString As String
    Private _groupbySettings As GridGroupedColumnCollection
    Private _gridColumns As GridColumnCollection
    Private _selectedviewIndex As Integer
    Private _selectedviewName As String
    Private _pageIndex As Integer
    Private _pageSize As Integer
    Private _recordCount As Integer
#End Region

#Region " Constructors "
    Public Sub New()
        Me.New(DEF_ID, DEF_TYPE, DEF_RECORD_TYPE, DEF_PAGE_INDEX, DEF_PAGE_SIZE)
    End Sub

    Public Sub New(ByVal sId As String, ByVal gtGridType As GridType, ByVal recordType As Integer, ByVal pageIndex As Integer, ByVal pageSize As Integer)
        MyBase.New()

        _id = String.Empty
        _type = GridType.NotSet
        _recordType = 0
        _selectString = String.Empty
        _tableName = String.Empty
        _whereString = String.Empty
        _orderbyString = String.Empty
        _groupbyString = String.Empty
        _hiddenfieldsString = String.Empty
        _groupbySettings = Nothing
        _selectedviewIndex = 0
        _selectedviewName = String.Empty
        _pageIndex = DEF_PAGE_INDEX
        _pageSize = DEF_PAGE_SIZE
        _recordCount = 0

    End Sub

    Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
        MyBase.New(info, context)

        _id = info.GetString(PROP_GRIDID_STRING)
        _type = CType(info.GetInt32(PROP_GRIDTYPE_STRING), GridType)
        _recordType = info.GetInt32(PROP_RECORDTYPE_STRING)
        _selectString = info.GetString(PROP_SELECT_STRING)
        _tableName = info.GetString(PROP_TABLE_NAME)
        _whereString = info.GetString(PROP_WHERE_STRING)
        _orderbyString = info.GetString(PROP_ORDERBY_STRING)
        _groupbyString = info.GetString(PROP_GROUPBY_STRING)
        _hiddenfieldsString = info.GetString(PROP_HIDDENFIELDS_STRING)
        Dim tempColumnCollection As New GridGroupedColumnCollection
        _groupbySettings = CType(info.GetValue(PROP_GROUPBY_SETTINGS_STRING, tempColumnCollection.GetType()), GridGroupedColumnCollection)
        _selectedviewIndex = info.GetInt32(PROP_SELECTEDVIEWINDEX_STRING)
        _selectedviewName = info.GetString(PROP_SELECTEDVIEWNAME_STRING)
        _pageIndex = info.GetInt32(PROP_PAGEINDEX_STRING)
        _pageSize = info.GetInt32(PROP_PAGESIZE_STRING)
        _recordCount = info.GetInt32(PROP_RECORDCOUNT_STRING)

    End Sub
#End Region

#Region " Properties "
    Public Property SelectString() As String
        Get
            Return Me._selectString
        End Get
        Set(value As String)
            Me._selectString = value
        End Set
    End Property

    Public Property TableName() As String
        Get
            Return Me._tableName
        End Get
        Set(value As String)
            Me._tableName = value
        End Set
    End Property

    Public Property WhereString() As String
        Get
            Return Me._whereString
        End Get
        Set(value As String)
            Me._whereString = value
        End Set
    End Property

    Public Property OrderByString() As String
        Get
            Return _orderbyString
        End Get
        Set(value As String)
            Me._orderbyString = value
        End Set
    End Property

    Public Property GroupByString() As String
        Get
            Return _groupbyString
        End Get
        Set(value As String)
            Me._groupbyString = value
        End Set
    End Property

    Default Public Property GridId(ByVal sId As String) As String
        Get
            Return _id
        End Get
        Set(value As String)
            Me._id = value
        End Set
    End Property

    Public Property GridType() As GridType
        Get
            Return _type
        End Get
        Set(value As GridType)
            Me._type = value
        End Set
    End Property

    Public Property RecordType() As Integer
        Get
            Return _recordType
        End Get
        Set(value As Integer)
            Me._recordType = value
        End Set
    End Property

    Public Property HiddenFields() As String
        Get
            Return _hiddenfieldsString
        End Get
        Set(value As String)
            Me._hiddenfieldsString = value
        End Set
    End Property

    Public Property GroupedColumnSettings() As GridGroupedColumnCollection
        Get
            Return Me._groupbySettings
        End Get
        Set(value As GridGroupedColumnCollection)
            Me._groupbySettings = value
        End Set
    End Property

    Public Property GridColumns() As GridColumnCollection
        Get
            Return Me._gridColumns
        End Get
        Set(value As GridColumnCollection)
            Me._gridColumns = value
        End Set
    End Property

    Public Property SelectedViewIndex() As Integer
        Get
            Return Me._selectedviewIndex
        End Get
        Set(value As Integer)
            Me._selectedviewIndex = value
        End Set
    End Property

    Public Property SelectedViewName() As String
        Get
            Return Me._selectedviewName
        End Get
        Set(value As String)
            Me._selectedviewName = value
        End Set
    End Property

    Public Property PageIndex() As Integer
        Get
            Return Me._pageIndex
        End Get
        Set(value As Integer)
            Me._pageIndex = value
        End Set
    End Property

    Public Property PageSize() As Integer
        Get
            Return Me._pageSize
        End Get
        Set(value As Integer)
            Me._pageSize = value
        End Set
    End Property

    Public Property RecordCount() As Integer
        Get
            Return Me._recordCount
        End Get
        Set(value As Integer)
            Me._recordCount = value
        End Set
    End Property
#End Region

#Region " Methods "
    Public Overloads Sub Initialize(ByVal gridID As String, ByVal recordType As Integer, _
                                    ByVal selectString As String, tableName As String,
                                    ByVal whereString As String, ByVal orderbyString As String, _
                                    ByVal groupbyString As String, ByVal hiddenFields As String, _
                                    ByVal gridType As GridType, _
                                    ByVal groupedColumnSettings As GridGroupedColumnCollection, ByVal selectedViewIndex As Integer, _
                                    ByVal selectedViewName As String, ByVal pageIndex As Integer, _
                                    ByVal pageSize As Integer, ByVal recordCount As Integer)
        Try
            _id = gridID
        Catch ex As Exception
            _id = String.Empty
        End Try
        Try
            _recordType = recordType
        Catch ex As Exception
            _recordType = 0
        End Try
        Try
            _type = gridType
        Catch ex As Exception
            _type = Grid.GridType.NotSet
        End Try
        Try
            _selectString = selectString
        Catch ex As Exception
            _selectString = String.Empty
        End Try
        Try
            _tableName = tableName
        Catch ex As Exception
            _tableName = String.Empty
        End Try
        Try
            _whereString = whereString
        Catch ex As Exception
            _whereString = String.Empty
        End Try
        Try
            _orderbyString = orderbyString
        Catch ex As Exception
            _orderbyString = String.Empty
        End Try
        Try
            _groupbyString = groupbyString
        Catch ex As Exception
            _groupbyString = String.Empty
        End Try
        Try
            _hiddenfieldsString = hiddenFields
        Catch ex As Exception
            _hiddenfieldsString = String.Empty
        End Try
        Try
            _groupbySettings = groupedColumnSettings
        Catch ex As Exception
            _groupbySettings = Nothing
        End Try
        Try
            _selectedviewIndex = selectedViewIndex
        Catch ex As Exception
            _selectedviewIndex = 0
        End Try
        Try
            _selectedviewName = selectedViewName
        Catch ex As Exception
            _selectedviewName = String.Empty
        End Try
        Try
            _pageIndex = pageIndex
        Catch ex As Exception
            _pageIndex = DEF_PAGE_INDEX
        End Try
        Try
            _pageSize = pageSize
        Catch ex As Exception
            _pageSize = DEF_PAGE_SIZE
        End Try
        Try
            _recordCount = recordCount
        Catch ex As Exception
            _recordCount = 0
        End Try
    End Sub

#Region " Grid Data "
    Public Shared Sub InitializeGridView(ByVal iView As Integer, ByRef dgs As Grid.Settings)

        If IsNothing(iView) Then
            Throw New ArgumentNullException("iView")
        End If
        If IsNothing(dgs) Then
            Throw New ArgumentNullException("dgs")
        End If

        Dim strSelect As String = String.Empty
        Dim strGroupBy As String = String.Empty
        Select Case CInt(iView)
            Case enumViews.DEFAULT_LIST
                strSelect = Views.DEFAULT_LIST
            Case enumViews.BU_NATURE_OF_RECORD
                strSelect = Views.BU_NATURE_OF_RECORD
                strGroupBy = "Ref Ascending"
            Case enumViews.DIRECTORATE_PROJECTS
                strSelect = Views.DIRECTORATE_PROJECTS
            Case enumViews.MY_RECORDS
                strSelect = Views.MY_RECORDS
            Case enumViews.PROJECTS
                strSelect = Views.PROJECTS
            Case enumViews.RED_RECORDS
                strSelect = Views.RED_RECORDS
            Case enumViews.RECORDS_BY_BUS_PROCESS
                strSelect = Views.RECORDS_BY_BUS_PROCESS
            Case enumViews.RECORDS_BY_CATEGORY
                strSelect = Views.RECORDS_BY_CATEGORY
            Case enumViews.RECORDS_BY_NATURE
                strSelect = Views.RECORDS_BY_NATURE
            Case enumViews.RECORDS_BY_TYPE
                strSelect = Views.RECORDS_BY_TYPE
        End Select

        dgs.SelectString = dgs.GetSqlFields(strSelect)
        dgs.GroupByString = dgs.GetSqlGroupBy(strGroupBy)
        dgs.HiddenFields = Data.HiddenFields.DEFAULT_LIST

    End Sub

    ''' <summary>
    ''' Returns an SQL field list including fields from HiddenFields and Fields.
    ''' </summary>
    Public Function GetSqlFields(strFields As String) As String
        Dim fields As New FieldCollection
        fields.Add(strFields)
        Return fields.ToSql()
    End Function

    ''' <summary>
    ''' Returns an SQL filter string including filters from <see cref="Filter">Filter</see>
    ''' </summary>
    Public Function GetSqlWhere(strWhere As String) As String
        'Dim scSelectionCriteria As New WhereClause(strTableName, strFieldName, ComparisonOperator.EqualTo, strRiskStatusFlag, OdbcType.VarChar)
        'Dim whereCriteria As New WhereCollection(scSelectionCriteria)

        'whereCriteria.ToString()

        'Return whereCriteria.ToString()
        Return String.Empty
    End Function

    ''' <summary>
    ''' Returns an Sql group criteria string.
    ''' </summary>
    Public Function GetSqlGroupBy(strGroupBy As String) As String
        Dim groupCriteria As New GroupByCollection
        groupCriteria.FromSql(strGroupBy)
        strGroupBy = groupCriteria.ToSql()

        Return strGroupBy
    End Function

    ''' <summary>
    ''' Returns an Sql sort criteria string.
    ''' </summary>
    Public Function GetSqlOrderBy(strOrderBy As String) As String
        If Not String.IsNullOrEmpty(strOrderBy) Then
            If HasSortingCriteria(strOrderBy) Then
                Dim sortCriteria As New OrderByCollection
                sortCriteria.FromSql(strOrderBy)
                strOrderBy = sortCriteria.ToSql()
            Else
                strOrderBy = String.Format("{1}{0}{2}", strOrderBy, Keyword.OpenQuote, Keyword.CloseQuote)
            End If
        End If
        Return strOrderBy
    End Function

    Public Function GroupedColumnNames(jcGroupedColumns As GroupByCollection) As String
        Dim grpCols As String = ""

        For i As Integer = 0 To jcGroupedColumns.Count - 1
            If (jcGroupedColumns(i).ToString.EndsWith(" (ascending)")) Then
                grpCols = Field.GetFieldName(jcGroupedColumns(i).ToString.Substring(0, jcGroupedColumns(i).ToString.Length - 12)) & ","
            ElseIf (jcGroupedColumns(i).ToString.EndsWith(" ASC")) Then
                grpCols = Field.GetFieldName(jcGroupedColumns(i).ToString.Substring(0, jcGroupedColumns(i).ToString.Length - 4)) & ","
            ElseIf (jcGroupedColumns(i).ToString.EndsWith(" ASCENDING")) Then
                grpCols = Field.GetFieldName(jcGroupedColumns(i).ToString.Substring(0, jcGroupedColumns(i).ToString.Length - 10)) & ","
            ElseIf (jcGroupedColumns(i).ToString.EndsWith(" (descending)")) Then
                grpCols = Field.GetFieldName(jcGroupedColumns(i).ToString.Substring(0, jcGroupedColumns(i).ToString.Length - 13)) & ","
            ElseIf (jcGroupedColumns(i).ToString.EndsWith(" DESC")) Then
                grpCols = Field.GetFieldName(jcGroupedColumns(i).ToString.Substring(0, jcGroupedColumns(i).ToString.Length - 5)) & ","
            ElseIf (jcGroupedColumns(i).ToString.EndsWith(" DESCENDING")) Then
                grpCols = Field.GetFieldName(jcGroupedColumns(i).ToString.Substring(0, jcGroupedColumns(i).ToString.Length - 11)) & ","
            Else
                grpCols = Field.GetFieldName(jcGroupedColumns(i).ToString) & ","
            End If
        Next
        grpCols = grpCols.Substring(0, grpCols.LastIndexOf(","))
        Return grpCols
    End Function

    Public Function GroupColumnNames(strGroupedColumns As String) As String()
        If IsNothing(strGroupedColumns) Then
            Return Nothing
        End If
        Dim aColumnNames() As String = strGroupedColumns.Split(Keyword.FieldDelimiter)
        Dim grpCols(aColumnNames.Count - 1) As String
        If strGroupedColumns <> String.Empty Then
            For i As Integer = 0 To aColumnNames.Count - 1
                If (aColumnNames(i).ToString.EndsWith(" (ascending)")) Then
                    grpCols(i) = Field.GetFieldName(aColumnNames(i).ToString.Substring(0, aColumnNames(i).ToString.Length - 12))
                ElseIf (aColumnNames(i).ToString.EndsWith(" ASC")) Then
                    grpCols(i) = Field.GetFieldName(aColumnNames(i).ToString.Substring(0, aColumnNames(i).ToString.Length - 4))
                ElseIf (aColumnNames(i).ToString.EndsWith(" ASCENDING")) Then
                    grpCols(i) = Field.GetFieldName(aColumnNames(i).ToString.Substring(0, aColumnNames(i).ToString.Length - 10))
                ElseIf (aColumnNames(i).ToString.EndsWith(" (descending)")) Then
                    grpCols(i) = Field.GetFieldName(aColumnNames(i).ToString.Substring(0, aColumnNames(i).ToString.Length - 13))
                ElseIf (aColumnNames(i).ToString.EndsWith(" DESC")) Then
                    grpCols(i) = Field.GetFieldName(aColumnNames(i).ToString.Substring(0, aColumnNames(i).ToString.Length - 5))
                ElseIf (aColumnNames(i).ToString.EndsWith(" DESCENDING")) Then
                    grpCols(i) = Field.GetFieldName(aColumnNames(i).ToString.Substring(0, aColumnNames(i).ToString.Length - 11))
                Else
                    grpCols(i) = Field.GetFieldName(aColumnNames(i).ToString)
                End If
            Next
            Return grpCols
        Else
            Return Nothing
        End If
    End Function

    Public Function ConvertSortDesciptionToEnum(ByVal sortDescription As String) As Integer
        Select Case sortDescription
            Case "ascending"
                Return SortIndicator.Ascending
            Case "Ascending"
                Return SortIndicator.Ascending
            Case "ASC"
                Return SortIndicator.Ascending
            Case "ASCENDING"
                Return SortIndicator.Ascending
            Case "descending"
                Return SortIndicator.Descending
            Case "Descending"
                Return SortIndicator.Descending
            Case "DESC"
                Return SortIndicator.Descending
            Case "DESCENDING"
                Return SortIndicator.Descending
            Case Else
                Return SortIndicator.None
        End Select
    End Function

    Private Function HasSortingCriteria(strGroupedColumns As String) As Boolean
        Dim bFlag As Boolean = False
        If IsNothing(strGroupedColumns) Then
            bFlag = False
        End If
        Dim aColumnNames() As String = strGroupedColumns.Split(Keyword.FieldDelimiter)
        If strGroupedColumns <> String.Empty Then
            For i As Integer = 0 To aColumnNames.Count - 1
                If (aColumnNames(i).ToString.EndsWith(" (ascending)")) Then
                    bFlag = True
                ElseIf (aColumnNames(i).ToString.EndsWith(" 1")) Then
                    bFlag = True
                ElseIf (aColumnNames(i).ToString.EndsWith(" ASC")) Then
                    bFlag = True
                ElseIf (aColumnNames(i).ToString.EndsWith(" ASCENDING")) Then
                    bFlag = True
                ElseIf (aColumnNames(i).ToString.EndsWith(" (descending)")) Then
                    bFlag = True
                ElseIf (aColumnNames(i).ToString.EndsWith(" 2")) Then
                    bFlag = True
                ElseIf (aColumnNames(i).ToString.EndsWith(" DESC")) Then
                    bFlag = True
                ElseIf (aColumnNames(i).ToString.EndsWith(" DESCENDING")) Then
                    bFlag = True
                Else
                    bFlag = False
                End If
            Next
        Else
            bFlag = False
        End If
        Return bFlag
    End Function
#End Region

#Region " Grid Layout "
    ''' <summary>
    ''' Returns a string to be used as part of a grid cell image background CSS style class.
    ''' </summary>
    Public Shared Function create_ImageCssClass() As String
        Dim sClass As String = "padding-left:4px;" & _
                               "padding-right:2px;" & _
                               "padding-top:2px;" & _
                               "padding-bottom:2px;" & _
                               "border-bottom:solid 1px #D8D8D8;" & _
                               "background-position:center;" & _
                               "background-repeat:no-repeat; "
        Return sClass
    End Function

    ''' <summary>
    ''' Gets an image path for a risk profile.
    ''' </summary>
    Public Shared Function get_ProfileImagePath(ByVal sProfileCode As String) As String
        Dim strImagePath As String = String.Empty
        Select Case (sProfileCode)
            Case "L"
                strImagePath = String.Format("images/profile/{0}", "FF00.gif")
            Case "M"
                strImagePath = String.Format("images/profile/{0}", "FFFF.gif")
            Case "H"
                strImagePath = String.Format("images/profile/{0}", "FF.gif")
        End Select

        Return strImagePath
    End Function
#End Region

#Region " Hierarchical Data Grid"
    ''' <summary>
    ''' Initialize the hierarchical grid columns
    ''' </summary>
    Public Overloads Shared Sub Initialize_HierarchicalGridColumns(ByRef grid As WebHierarchicalDataGrid, ByRef dgSettings As Grid.Settings, _
                                                                   ByVal gridType As GridType)
        If grid.Columns.Count > 0 Then 'If grid.GridView.Columns.Count > 0 Then
            grid.Columns.Clear() 'grid.GridView.Columns.Clear()
        End If

        'Initialize Hidden Columns
        Dim hidCols As New GridColumnCollection(dgSettings.HiddenFields, True)
        For Each hidcol As GridColumn In hidCols
            Initialize_HierarchicalGridColumn(grid, hidcol) ' Initialize_HierarchicalGridColumn(grid.GridView, hidcol)
            Format_HierarchicalGridColumn(grid, hidcol, gridType, hidcol.Column.Contains(HIDDEN_FIELDNAME_PREFIX.Trim)) ' Format_HierarchicalGridColumn(grid.GridView, hidcol, gridType, hidcol.Column.Contains(HIDDEN_FIELDNAME_PREFIX.Trim))
        Next

        'Initialize Visible Columns
        If IsNothing(dgSettings.GridColumns) Then
            Dim gcCollection As New GridColumnCollection(dgSettings.SelectString)
            dgSettings.GridColumns = gcCollection
        End If
        If (dgSettings.GridColumns.Count <= 0) Then
            Dim gcCollection As New GridColumnCollection(dgSettings.SelectString)
            dgSettings.GridColumns = gcCollection
        End If
        For Each column As GridColumn In dgSettings.GridColumns
            Initialize_HierarchicalGridColumn(grid, column) ' Initialize_HierarchicalGridColumn(grid.GridView, column)
            Format_HierarchicalGridColumn(grid, column, gridType, column.Column.Contains(HIDDEN_FIELDNAME_PREFIX.Trim)) ' Format_HierarchicalGridColumn(grid.GridView, column, gridType, column.Column.Contains(HIDDEN_FIELDNAME_PREFIX.Trim))
            'Initialize_HierarchicalGridColumnSettings(grid.GridView, column.Column, column.Column)
            'Initialize_HierarchicalGridColumnGroupingSettings(grid, column.Column, column.Column)
        Next

        'Initialize GroupBy Columns
        Dim grpdCols As New GridColumnCollection(dgSettings.GroupedColumnSettings)
        For Each grpdCol As GridColumn In grpdCols
            If Not dgSettings.GridColumns.Contains(grpdCol) Then
                Initialize_HierarchicalGridColumn(grid, grpdCol) ' Initialize_HierarchicalGridColumn(grid.GridView, grpdCol)
                Format_HierarchicalGridColumn(grid, grpdCol, gridType, grpdCol.Column.Contains(HIDDEN_FIELDNAME_PREFIX.Trim)) ' Format_HierarchicalGridColumn(grid.GridView, grpdCol, gridType, grpdCol.Column.Contains(HIDDEN_FIELDNAME_PREFIX.Trim))
            End If
        Next
    End Sub

    ''' <summary>
    ''' Initializes the hierachical grid columns
    ''' </summary>
    Private Overloads Shared Sub Initialize_HierarchicalGridColumn(ByRef band As IBand, ByVal dataField As String, ByVal dataType As String, _
                                                                   ByVal key As String, ByVal Caption As String, ByVal hidden As Boolean)
        If Caption <> FieldName.Completed Then
            Dim column As New BoundDataField()
            column.Key = key & "".ToString()
            column.DataType = dataType & "".ToString
            column.DataFieldName = dataField & "".ToString()
            Select Case Caption
                Case FieldName.CostToDate
                    column.DataFormatString = "{0:c}"
                Case FieldName.EstimatedCost
                    column.DataFormatString = "{0:c}"
                Case FieldName.DueDate, FieldName.CompletedDate, FieldName.AmendedDate
                    column.DataFormatString = "{0:d}"
                Case Else

            End Select
            band.Columns.Add(column)
        Else
            Dim column As New BoundCheckBoxField
            column.Key = key & "".ToString()
            column.DataType = dataType & "".ToString
            column.DataFieldName = dataField & "".ToString
            column.CheckBox.CheckedImageUrl = IMAGES_WEBTAB_PATH & "Checked.gif"
            column.CheckBox.PartialImageUrl = IMAGES_WEBTAB_PATH & "Checked.gif"
            column.CheckBox.UncheckedImageUrl = IMAGES_WEBTAB_PATH & "Unchecked.gif"
            column.CssClass = CssClass.WHDGGridCheckboxCell
            band.Columns.Add(column)
        End If
    End Sub

    Private Overloads Shared Sub Initialize_HierarchicalGridColumn(ByRef band As IBand, ByRef column As GridColumn)
        If column.Column <> FieldName.Completed Then
            Dim col As New BoundDataField()
            col.Key = column.Column
            col.DataType = column.ColumnDataType
            col.DataFieldName = column.Column
            Select Case column.Column
                Case FieldName.CostToDate
                    col.DataFormatString = "{0:c}"
                Case FieldName.EstimatedCost
                    col.DataFormatString = "{0:c}"
                Case FieldName.DueDate, FieldName.CompletedDate, FieldName.AmendedDate
                    col.DataFormatString = "{0:d}"
                Case Else

            End Select
            band.Columns.Add(col)
        Else
            Dim col As New BoundCheckBoxField
            col.Key = column.Column
            col.DataType = column.ColumnDataType
            col.DataFieldName = column.Column
            col.CheckBox.CheckedImageUrl = IMAGES_WEBTAB_PATH & "Checked.gif"
            col.CheckBox.PartialImageUrl = IMAGES_WEBTAB_PATH & "Checked.gif"
            col.CheckBox.UncheckedImageUrl = IMAGES_WEBTAB_PATH & "Unchecked.gif"
            col.CssClass = CssClass.WHDGGridCheckboxCell
            band.Columns.Add(col)
        End If
    End Sub

    ''' <summary>
    ''' Format the hierachical grid columns
    ''' </summary>
    Private Overloads Shared Sub Format_HierarchicalGridColumn(ByRef band As IBand, ByVal key As String, ByVal Caption As String, _
                                                               ByVal hidden As Boolean, ByVal gridType As GridType)

        band.Columns(key).Hidden = hidden
        band.Columns(key).Header.Text = Caption
        Select Case Caption
            Case FieldName.Alert
                band.Columns(key).Header.CssClass = CssClass.WHDGGridHeaderAlert
                band.Columns(key).Header.Text = String.Empty
                band.Columns(key).Header.Tooltip = Caption
                band.Columns(key).Width = Unit.Pixel(DEF_IMG_COLUMN_WIDTH)
            Case FieldName.Completed
                band.Columns(key).Header.CssClass = CssClass.WHDGGridHeaderCompleted
                band.Columns(key).Header.Text = String.Empty
                band.Columns(key).Header.Tooltip = Caption
            Case FieldName.DueDate, FieldName.CompletedDate, FieldName.AmendedDate
                band.Columns(key).Header.Tooltip = Caption
                band.Columns(key).Width = Unit.Pixel(DEF_IMG_COLUMN_WIDTH)
            Case FieldName.CurrentProfile
                band.Columns(key).Header.CssClass = CssClass.WHDGGridHeaderCurrentProfile
                band.Columns(key).Header.Text = String.Empty
                band.Columns(key).Header.Tooltip = Caption
                band.Columns(key).Width = Unit.Pixel(DEF_IMG_COLUMN_WIDTH)
            Case FieldName.ControlledProfile
                band.Columns(key).Header.CssClass = CssClass.WHDGGridHeaderControlledProfile
                band.Columns(key).Header.Text = String.Empty
                band.Columns(key).Header.Tooltip = Caption
                band.Columns(key).Width = Unit.Pixel(DEF_IMG_COLUMN_WIDTH)
            Case FieldName.UncontrolledProfile
                band.Columns(key).Header.CssClass = CssClass.WHDGGridHeaderUncontrolledProfile
                band.Columns(key).Header.Text = String.Empty
                band.Columns(key).Header.Tooltip = Caption
                band.Columns(key).Width = Unit.Pixel(DEF_IMG_COLUMN_WIDTH)
            Case Else
                band.Columns(key).Header.Tooltip = Caption
                band.Columns(key).Width = Unit.Pixel(DEF_COLUMN_WIDTH)
        End Select
    End Sub

    Private Overloads Shared Sub Format_HierarchicalGridColumn(ByRef band As IBand, ByVal column As GridColumn, _
                                                               ByVal gridType As GridType, ByVal hidden As Boolean)
        band.Columns(column.Column).Hidden = hidden
        Select Case column.Column
            Case FieldName.Alert
                band.Columns(column.Column).Header.CssClass = CssClass.WHDGGridHeaderAlert
                band.Columns(column.Column).Header.Text = String.Empty
                band.Columns(column.Column).Header.Tooltip = column.Column
                If (IsNothing(column.ColumnWidth)) Or (CStr(column.ColumnWidth) = String.Empty) Or (column.ColumnWidth = 0) Then
                    band.Columns(column.Column).Width = Unit.Pixel(DEF_IMG_COLUMN_WIDTH)
                Else
                    band.Columns(column.Column).Width = Unit.Pixel(column.ColumnWidth)
                End If
            Case FieldName.Completed
                band.Columns(column.Column).Header.CssClass = CssClass.WHDGGridHeaderCompleted
                band.Columns(column.Column).Header.Text = String.Empty
                band.Columns(column.Column).Header.Tooltip = column.Column
                If IsNothing(column.ColumnWidth) Then
                    band.Columns(column.Column).Width = Unit.Pixel(DEF_IMG_COLUMN_WIDTH)
                Else
                    band.Columns(column.Column).Width = Unit.Pixel(column.ColumnWidth)
                End If
            Case FieldName.DueDate, FieldName.CompletedDate, FieldName.AmendedDate
                band.Columns(column.Column).Header.Tooltip = column.Column
                If (IsNothing(column.ColumnWidth)) Or (CStr(column.ColumnWidth) = String.Empty) Or (column.ColumnWidth = 0) Then
                    band.Columns(column.Column).Width = Unit.Pixel(DEF_IMG_COLUMN_WIDTH)
                Else
                    band.Columns(column.Column).Width = Unit.Pixel(column.ColumnWidth)
                End If
            Case FieldName.CurrentProfile
                band.Columns(column.Column).Header.CssClass = CssClass.WHDGGridHeaderCurrentProfile
                band.Columns(column.Column).Header.Text = String.Empty
                band.Columns(column.Column).Header.Tooltip = column.Column
                If (IsNothing(column.ColumnWidth)) Or (CStr(column.ColumnWidth) = String.Empty) Or (column.ColumnWidth = 0) Then
                    band.Columns(column.Column).Width = Unit.Pixel(DEF_IMG_COLUMN_WIDTH)
                Else
                    band.Columns(column.Column).Width = Unit.Pixel(column.ColumnWidth)
                End If
            Case FieldName.ControlledProfile
                band.Columns(column.Column).Header.CssClass = CssClass.WHDGGridHeaderControlledProfile
                band.Columns(column.Column).Header.Text = String.Empty
                band.Columns(column.Column).Header.Tooltip = column.Column
                If (IsNothing(column.ColumnWidth)) Or (CStr(column.ColumnWidth) = String.Empty) Or (column.ColumnWidth = 0) Then
                    band.Columns(column.Column).Width = Unit.Pixel(DEF_IMG_COLUMN_WIDTH)
                Else
                    band.Columns(column.Column).Width = Unit.Pixel(column.ColumnWidth)
                End If
            Case FieldName.UncontrolledProfile
                band.Columns(column.Column).Header.CssClass = CssClass.WHDGGridHeaderUncontrolledProfile
                band.Columns(column.Column).Header.Text = String.Empty
                band.Columns(column.Column).Header.Tooltip = column.Column
                If (IsNothing(column.ColumnWidth)) Or (CStr(column.ColumnWidth) = String.Empty) Or (column.ColumnWidth = 0) Then
                    band.Columns(column.Column).Width = Unit.Pixel(DEF_IMG_COLUMN_WIDTH)
                Else
                    band.Columns(column.Column).Width = Unit.Pixel(column.ColumnWidth)
                End If
            Case Else
                band.Columns(column.Column).Header.Text = column.Column
                band.Columns(column.Column).Header.Tooltip = column.Column
                If (IsNothing(column.ColumnWidth)) Or (CStr(column.ColumnWidth) = String.Empty) Or (column.ColumnWidth = 0) Then
                    band.Columns(column.Column).Width = Unit.Pixel(DEF_COLUMN_WIDTH)
                Else
                    band.Columns(column.Column).Width = Unit.Pixel(column.ColumnWidth)
                End If
        End Select
    End Sub

    ''' <summary>
    ''' Initialize hierarchical grid column setting(s)
    ''' </summary>
    Private Shared Sub Initialize_HierarchicalGridColumnSettings(ByRef band As IBand, ByVal key As String, ByVal Caption As String)
        Dim colEditSetting As New EditingColumnSetting
        Dim colMoveSetting As New ColumnMoveSetting
        Dim colSortSetting As New SortingColumnSetting
        Dim colResizeSetting As New ColumnResizeSetting

        colEditSetting.ColumnKey = key
        colEditSetting.ReadOnly = True
        band.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings.Add(colEditSetting)
        colMoveSetting.ColumnKey = key
        colMoveSetting.EnableMove = True
        band.Behaviors.ColumnMoving.ColumnSettings.Add(colMoveSetting)
        colSortSetting.ColumnKey = key
        colSortSetting.Sortable = True
        band.Behaviors.Sorting.ColumnSettings.Add(colSortSetting)
        colResizeSetting.ColumnKey = key
        colResizeSetting.EnableResize = True
        band.Behaviors.ColumnResizing.ColumnSettings.Add(colResizeSetting)

    End Sub

    ''' <summary>
    ''' Initialize hierarchical grid column grouping setting(s)
    ''' </summary>
    Private Shared Sub Initialize_HierarchicalGridColumnGroupingSettings(ByRef Grid As WebHierarchicalDataGrid, ByVal key As String, ByVal Caption As String)
        Dim colGroupingSetting As New ColumnGroupingSetting
        Select Case Caption
            Case FieldName.Completed
                colGroupingSetting.ColumnKey = key
                colGroupingSetting.Groupable = False
                Grid.GroupingSettings.ColumnSettings.Add(colGroupingSetting)
            Case Else
        End Select
    End Sub

    ''' <summary>
    ''' Initialize hierarchical grid column grouping.
    ''' </summary>
    Public Shared Sub Initialize_HierarchicalGridGrouping(ByRef grid As WebHierarchicalDataGrid, ByRef gridSettings As Grid.Settings)
        Dim newGroupCollection As New GridGroupedColumnCollection

        If gridSettings.GroupByString <> String.Empty Then
            Dim groupCriteria As New GridGroupedColumnCollection(gridSettings.GroupByString)
            For Each column As GridGroupedColumn In groupCriteria
                If Not (gridSettings.GroupedColumnSettings.Contains(column)) Then
                    newGroupCollection.Add(column)
                End If
            Next
        End If

        If (newGroupCollection.Count <= 0) Then
            grid.GroupingSettings.GroupedColumns.Clear()
        Else
            For Each column As GridGroupedColumn In newGroupCollection
                grid.GroupingSettings.GroupedColumns.Add(column.Column, gridSettings.ConvertSortDesciptionToEnum(column.SortDirection))
            Next
        End If
        gridSettings.GroupedColumnSettings = newGroupCollection
    End Sub

    ''' <summary>
    ''' Initialize hierarchical grid rows
    ''' </summary>
    Public Shared Sub Initialize_HierarchicalGridRow(ByRef e As Infragistics.Web.UI.GridControls.RowEventArgs)
        Dim strRowCssClass As String = e.Row.CssClass
        If (e.Row.Items.FindItemByKey(FieldName.HiddenDaysOverdue) IsNot Nothing) Then ' Test if record is over due
            If (CInt(e.Row.Items.FindItemByKey(FieldName.HiddenDaysOverdue).Value) > 0) Then
                strRowCssClass = strRowCssClass.Trim & " " & CssClass.WHDGGridRowOverDue
            End If
        End If
        If (e.Row.Items.FindItemByKey(FieldName.HiddenCompleted) IsNot Nothing) Then ' Test if record is completed
            If (CBool(e.Row.Items.FindItemByKey(FieldName.HiddenCompleted).Value)) Then
                strRowCssClass = strRowCssClass.Trim & " " & CssClass.WHDGGridRowCompleted
            End If
        End If

        If strRowCssClass.Trim = String.Empty Then
            e.Row.CssClass = CssClass.WHDGGridRow
        Else
            e.Row.CssClass = strRowCssClass.Trim
        End If

        If e.Row.Items.Count > 0 Then 'Make sure there are columns to process
            For i As Integer = 0 To e.Row.Items.Count - 1
                With e.Row.Items.Item(i) 'Iterate through row cells and apply CssClasses
                    Select Case .Column.Header.FieldKey
                        Case FieldName.Alert
                            Select Case (CInt(.Value))
                                Case Alert.Overdue
                                    .CssClass = CssClass.WHDGGridCellOverdue
                                    .Tooltip = ALERT_OVERDUE
                                    .Text = HttpUtility.HtmlDecode("&nbsp;")
                                Case Alert.Due
                                    .CssClass = CssClass.WHDGGridCellDue
                                    .Tooltip = ALERT_DUE
                                    .Text = HttpUtility.HtmlDecode("&nbsp;")
                                Case Alert.None
                                    .CssClass = CssClass.WHDGGridCellNotDue
                                    .Text = HttpUtility.HtmlDecode("&nbsp;")
                            End Select
                        Case FieldName.Completed
                            If CBool(.Value) Then
                                .Tooltip = CHECKBOX_COMPLETED
                            Else
                                .Tooltip = CHECKBOX_NOT_COMPLETED
                            End If
                        Case FieldName.DueDate
                            .CssClass = CssClass.WHDGGridCell
                            .Text = .Column.FormatValue(e.Row.Items.FindItemByKey(FieldName.DueDate).Value)
                        Case FieldName.CompletedDate
                            .CssClass = CssClass.WHDGGridCell
                            .Text = .Column.FormatValue(e.Row.Items.FindItemByKey(FieldName.CompletedDate).Value)
                        Case FieldName.AmendedDate
                            .CssClass = CssClass.WHDGGridCell
                            .Text = .Column.FormatValue(e.Row.Items.FindItemByKey(FieldName.AmendedDate).Value)
                        Case FieldName.CostToDate
                            .CssClass = CssClass.WHDGGridCell
                            .Text = .Column.FormatValue(e.Row.Items.FindItemByKey(FieldName.CostToDate).Value)
                        Case FieldName.EstimatedCost
                            .CssClass = CssClass.WHDGGridCell
                            .Text = .Column.FormatValue(e.Row.Items.FindItemByKey(FieldName.EstimatedCost).Value)
                        Case FieldName.CurrentProfile
                            Dim sClass As String = ""
                            sClass = AppStyling.AppStylingManager.Settings.CssRegistry.Add(create_ImageCssClass() & "background-image: url(""" & _
                                                                        get_ProfileImagePath(e.Row.Items.FindItemByKey(FieldName.HiddenCurrentProfile).Value.ToString) & """);", "tbody tr td.{0}", False)
                            .CssClass = sClass
                            .Text = HttpUtility.HtmlDecode("&nbsp;")
                        Case FieldName.ControlledProfile
                            Dim sClass As String = ""
                            sClass = AppStyling.AppStylingManager.Settings.CssRegistry.Add(create_ImageCssClass() & "background-image: url(""" & _
                                                                       get_ProfileImagePath(e.Row.Items.FindItemByKey(FieldName.HiddenControlledProfile).Value.ToString) & """);", "tbody tr td.{0}", False)
                            .CssClass = sClass
                            .Text = HttpUtility.HtmlDecode("&nbsp;")
                        Case FieldName.UncontrolledProfile
                            Dim sClass As String = ""
                            sClass = AppStyling.AppStylingManager.Settings.CssRegistry.Add(create_ImageCssClass() & "background-image: url(""" & _
                                                                       get_ProfileImagePath(e.Row.Items.FindItemByKey(FieldName.HiddenUncontrolledProfile).Value.ToString) & """);", "tbody tr td.{0}", False)
                            .CssClass = sClass
                            .Text = HttpUtility.HtmlDecode("&nbsp;")
                        Case Else
                            .CssClass = CssClass.WHDGGridCell
                            .Text = HttpUtility.HtmlEncode(.Text.Trim).Replace(vbCrLf, " ").Replace("<br/>", " ")
                            .Tooltip = HttpUtility.HtmlEncode(.Text.Trim).Replace(vbCrLf, " ").Replace("<br/>", " ")
                            If .Text = String.Empty Then
                                .Text = HttpUtility.HtmlDecode("&nbsp;")
                            End If
                    End Select
                    Select Case (.Column.Type)
                        Case GetType(System.Int16), _
                             GetType(System.Int32), _
                             GetType(System.Int64), _
                             GetType(System.Decimal), _
                             GetType(System.Double), _
                             GetType(System.Byte)
                            If .Text = "0" Then
                                .Text = HttpUtility.HtmlDecode("&nbsp;")
                            End If
                    End Select
                    If .Text = String.Empty Then
                        .Text = HttpUtility.HtmlDecode("&nbsp;")
                    End If
                End With
            Next
        End If
    End Sub
#End Region
#End Region

#Region " Overrides "
    Protected Overridable Overloads Sub Dispose(ByVal fDisposing As Boolean)
        Try
            If (fDisposing) Then
                ' Free managed objects
            End If
            ' Free unmanaged objects
            ' Set large fields to null
            _id = Nothing
            _type = Nothing
            _recordType = Nothing
            _selectString = Nothing
            _tableName = Nothing
            _whereString = Nothing
            _orderbyString = Nothing
            _groupbyString = Nothing
            _hiddenfieldsString = Nothing
            _groupbySettings = Nothing
            _selectedviewIndex = Nothing
            _selectedviewName = Nothing
            _pageIndex = Nothing
            _pageSize = Nothing
            _recordCount = Nothing
        Finally
            MyBase.Dispose(fDisposing)
        End Try
    End Sub

    <SecurityPermission(SecurityAction.Demand, SerializationFormatter:=True)> _
    Public Overrides Sub GetObjectData(ByVal info As System.Runtime.Serialization.SerializationInfo, _
                                       ByVal context As System.Runtime.Serialization.StreamingContext)

        ' Call base handler
        MyBase.GetObjectData(info, context)

        ' Add class variables
        info.AddValue(PROP_GRIDID_STRING, _id)
        info.AddValue(PROP_GRIDTYPE_STRING, _type)
        info.AddValue(PROP_RECORDTYPE_STRING, _recordType)
        info.AddValue(PROP_SELECT_STRING, _selectString)
        info.AddValue(PROP_TABLE_NAME, _tableName)
        info.AddValue(PROP_WHERE_STRING, _whereString)
        info.AddValue(PROP_ORDERBY_STRING, _orderbyString)
        info.AddValue(PROP_GROUPBY_STRING, _groupbyString)
        info.AddValue(PROP_HIDDENFIELDS_STRING, _hiddenfieldsString)
        info.AddValue(PROP_GROUPBY_SETTINGS_STRING, _groupbySettings)
        info.AddValue(PROP_SELECTEDVIEWINDEX_STRING, _selectedviewIndex)
        info.AddValue(PROP_SELECTEDVIEWNAME_STRING, _selectedviewName)
        info.AddValue(PROP_PAGEINDEX_STRING, _pageIndex)
        info.AddValue(PROP_PAGESIZE_STRING, _pageSize)
        info.AddValue(PROP_RECORDCOUNT_STRING, _recordCount)

    End Sub
#End Region
End Class
